/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protege.code.generator.wrapping;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import edu.stanford.smi.protege.model.Cls;
import edu.stanford.smi.protege.model.Slot;

/**
 * Code generator for a class.
 */
public class ClsCode {

    private Cls cls;


    public ClsCode(Cls cls) {
        this.cls = cls;
    }


    public String getJavaName() {
        return getValidJavaName(cls.getName());
    }


    /**
     * @return a List of RDFPropertyAtClassCodes
     * @see SlotAtClassCode
     */
    @SuppressWarnings("unchecked")
	public List<SlotAtClassCode> getSlotCodes(boolean transitive) {
        Set<Slot> properties = new HashSet<Slot>();
        List<SlotAtClassCode> codes = new ArrayList<SlotAtClassCode>();


        Set<Slot> relevantSlots = new HashSet<Slot>(cls.getTemplateSlots());

        for (Slot slot : relevantSlots) {
            if (slot.isSystem()) {
            	//skip system properties
            	continue;
            }
            properties.add(slot);
            SlotAtClassCode code = new SlotAtClassCode(cls, slot);
            codes.add(code);

            Collection<Slot> subSlots = slot.getSubslots();
            Iterator<Slot> sit = subSlots.iterator();
            while (sit.hasNext()) {
                Slot subSlot = sit.next();
                if (!properties.contains(subSlot)) {
                    codes.add(new SlotAtClassCode(cls, subSlot));
                    properties.add(subSlot);
                }
            }
        }
        Collections.sort(codes);
        return codes;
    }


    public static String getValidJavaName(String name) {
        for (int i = 1; i < name.length(); i++) {
            char c = name.charAt(i);
            if (!Character.isJavaIdentifierPart(c)) {
                name = name.replace(c, '_');
            }
        }
        return name;
    }
}
