/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protege.code.generator.wrapping;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;

import edu.stanford.smi.protege.model.Cls;
import edu.stanford.smi.protege.util.ApplicationProperties;

public class ProjectBasedJavaCodeGeneratorOptions implements EditableJavaCodeGeneratorOptions {

    public final static String ABSTRACT_MODE = "JavaCodeAbstract";

    public final static String FACTORY_CLASS_NAME = "JavaCodeFactoryClassName";

    public final static String FILE_NAME = "JavaCodeFileName";

    public final static String PACKAGE = "JavaCodePackage";

    public final static String SET_MODE = "JavaCodeSet";

    public final static String PREFIX_MODE = "JavaCodeUsePrefix";

    private Collection<Cls> clses = new ArrayList<Cls>();

    public boolean getAbstractMode() {
    	return ApplicationProperties.getBooleanProperty(ABSTRACT_MODE, false);
    }


    public String getFactoryClassName() {
    	return ApplicationProperties.getString(FACTORY_CLASS_NAME, "MyFactory");
    }


    public File getOutputFolder() {
    	String fileName= ApplicationProperties.getString(FILE_NAME, "");
        return new File(fileName);
    }


    public String getPackage() {
    	return ApplicationProperties.getString(PACKAGE);
    }


    public boolean getSetMode() {
    	return ApplicationProperties.getBooleanProperty(SET_MODE, false);
    }

    public void setAbstractMode(boolean value) {
    	ApplicationProperties.setBoolean(ABSTRACT_MODE, value);
    }


    public void setOutputFolder(File file) {
    	ApplicationProperties.setString(FILE_NAME, file == null ? null : file.getAbsolutePath());
    }


    public void setFactoryClassName(String value) {
    	ApplicationProperties.setString(FACTORY_CLASS_NAME, value);
    }


    public void setPackage(String value) {
    	ApplicationProperties.setString(PACKAGE, value);
    }


    public void setSetMode(boolean value) {
    	ApplicationProperties.setBoolean(SET_MODE, value);
    }


	public void setClses(Collection<Cls> clses) {
		this.clses = clses;
	}


	public Collection<Cls> getClses() {
		return clses;
	}

}
