/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protege.model.framestore.undo;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import edu.stanford.smi.protege.model.Cls;
import edu.stanford.smi.protege.model.Facet;
import edu.stanford.smi.protege.model.Slot;
import edu.stanford.smi.protege.model.framestore.FrameStore;

public class DeleteClsCommand extends DeleteFrameCommand {
    private Map<Slot, Collection> directTemplateSlotValues = new HashMap<Slot, Collection>();
    private Map<Slot, Map<Facet, Collection>> directTemplateSlotFacetValues = new HashMap<Slot, Map<Facet, Collection>>();
    
    public DeleteClsCommand(FrameStore delegate, Cls cls) {
        super(delegate, cls);
    }
    
    public Cls getCls() {
        return (Cls) getFrame();
    }
    
    protected void saveFrame() {
        super.saveFrame();
        saveCls();
    }
    
    protected void restoreFrame() {
        super.restoreFrame();
        restoreCls();
    }
    
    private void saveCls() {
        Cls cls = getCls();
        Set<Slot> templateSlots = getDelegate().getTemplateSlots(cls);
        for (Slot slot : templateSlots) {
            Collection values = getDelegate().getDirectTemplateSlotValues(cls, slot);
            if (values != null && !values.isEmpty()) {
                directTemplateSlotValues.put(slot, values);
            }
            Map<Facet, Collection> facetMap = new HashMap<Facet, Collection>();
            directTemplateSlotFacetValues.put(slot, facetMap);
            Set<Facet> facets  = getDelegate().getTemplateFacets(cls, slot);
            for (Facet facet : facets) {
                Collection facetValues = getDelegate().getDirectTemplateFacetValues(cls, slot, facet);
                if (facetValues != null && !facetValues.isEmpty()) {
                    facetMap.put(facet, facetValues);
                }
            }
        }
    }
    
    private void restoreCls() {
        Cls cls = getCls();
        for (Entry<Slot, Collection>  entry : directTemplateSlotValues.entrySet()) {
            getDelegate().setDirectTemplateSlotValues(cls, entry.getKey(), entry.getValue());
        }
        for (Entry<Slot, Map<Facet, Collection>> entry : directTemplateSlotFacetValues.entrySet()) {
            Slot slot = entry.getKey();
            for (Entry<Facet, Collection> facetEntry : entry.getValue().entrySet()) {
                getDelegate().setDirectTemplateFacetValues(cls, slot, facetEntry.getKey(), facetEntry.getValue());
            }
        }
    }

}
