/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protege.model.framestore.undo;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import edu.stanford.smi.protege.model.Cls;
import edu.stanford.smi.protege.model.Frame;
import edu.stanford.smi.protege.model.Slot;
import edu.stanford.smi.protege.model.framestore.FrameStore;

public class DeleteSlotCommand extends DeleteFrameCommand {
    
    private Map<Frame, Collection> directOwnSlotValues = new HashMap<Frame, Collection>();
    private Map<Cls, Collection> directTemplateSlotValues = new HashMap<Cls, Collection>();

    public DeleteSlotCommand(FrameStore delegate, Slot slot) {
        super(delegate, slot);
    }

    public Slot getSlot() {
        return (Slot) getFrame();
    }
    
    @Override
    protected void saveFrame() {
        super.saveFrame();
        saveSlot();
    }
    
    @Override
    protected void restoreFrame() {
        super.restoreFrame();
        restoreSlot();
    }
    
    private void saveSlot() {
        Slot slot = getSlot();
        for (Frame frame : getDelegate().getFramesWithAnyDirectOwnSlotValue(slot)) {
            Collection values = getDelegate().getDirectOwnSlotValues(frame, slot);
            if (values != null && !values.isEmpty()) {
                directOwnSlotValues.put(frame, values);
            }
        }
        for (Cls cls : getDelegate().getClsesWithAnyDirectTemplateSlotValue(slot)) {
            Collection values = getDelegate().getDirectTemplateSlotValues(cls, slot);
            if  (values != null && !values.isEmpty()) {
                directTemplateSlotValues.put(cls, values);
            }
        }
    }
    
    private  void restoreSlot() {
        for (Entry<Frame, Collection> entry : directOwnSlotValues.entrySet()) {
            getDelegate().setDirectOwnSlotValues(entry.getKey(), getSlot(), entry.getValue());
        }
        for (Entry<Cls, Collection> entry : directTemplateSlotValues.entrySet()) {
            getDelegate().setDirectTemplateSlotValues(entry.getKey(), getSlot(), entry.getValue());
        }
    }
}
