/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protege.model.framestore.undo;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.model.framestore.*;

class MoveDirectSubslotCommand extends AbstractCommand {
    private Slot slot;
    private int index;
    private Slot subslot;
    private int oldIndex;

    MoveDirectSubslotCommand(FrameStore delegate, Slot slot, int index, Slot subslot) {
        super(delegate);
        this.slot = slot;
        this.index = index;
        this.subslot = subslot;
        oldIndex = getDelegate().getDirectSubslots(slot).indexOf(subslot);
        setDescription("Move subslot " + getText(subslot) + " of slot " + getText(slot) + " to index " + index);
    }

    public Object doIt() {
        getDelegate().moveDirectSubslot(slot, subslot, index);
        return null;
    }

    public void undoIt() {
        getDelegate().moveDirectSubslot(slot, subslot, oldIndex);
    }

    public void redoIt() {
        getDelegate().moveDirectSubslot(slot, subslot, index);
    }
}
