/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protege.model.framestore.undo;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.model.framestore.*;

class MoveDirectTypeCommand extends AbstractCommand {
    private Instance instance;
    private int index;
    private Cls type;
    private int oldIndex;

    MoveDirectTypeCommand(FrameStore delegate, Instance instance, Cls type, int index) {
        super(delegate);
        this.instance = instance;
        this.index = index;
        this.type = type;
        oldIndex = getDelegate().getDirectTypes(instance).indexOf(type);
        setDescription("Move type " + getText(type) + " of instance " + getText(instance) + " to index " + index);
    }

    public Object doIt() {
        getDelegate().moveDirectType(instance, type, index);
        return null;
    }

    public void undoIt() {
        getDelegate().moveDirectType(instance, type, oldIndex);
    }

    public void redoIt() {
        getDelegate().moveDirectType(instance, type, index);
    }
}
