/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protege.server.admin;

import java.util.logging.Level;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;

import edu.stanford.smi.protege.model.KnowledgeBase;
import edu.stanford.smi.protege.model.Project;
import edu.stanford.smi.protege.server.RemoteProjectManager;
import edu.stanford.smi.protege.server.RemoteServer;
import edu.stanford.smi.protege.server.RemoteSession;
import edu.stanford.smi.protege.server.metaproject.MetaProject;
import edu.stanford.smi.protege.server.metaproject.impl.MetaProjectImpl;
import edu.stanford.smi.protege.util.Log;
import edu.stanford.smi.protege.widget.ClsesAndInstancesTab;

public class MetaprojectPanel extends AbstractRefreshableServerPanel {
    private static final long serialVersionUID = 6214203367190208198L;

    public MetaprojectPanel(RemoteServer server, RemoteSession session) {
        super(server, session);
    }

    @Override
    protected JComponent createCenterComponent() {
        MetaProject mp = null;
        try {
            mp = RemoteProjectManager.getInstance().connectToMetaProject(getServer(), getSession());
        } catch (Exception e) {
            Log.getLogger().log(Level.WARNING, "Error at getting the metaproject", e);
        }

        if (mp == null) {
            JPanel panel = new JPanel();
            panel.add(new JLabel("<html><i><b>You do not have enough privileges to change the metaproject.</b></i></html>"));
            panel.setBorder(BorderFactory.createRaisedBevelBorder());
            return panel;
        }

        KnowledgeBase mp_kb = ((MetaProjectImpl) mp).getKnowledgeBase();
        Project mp_prj = mp_kb.getProject();
        mp_prj.setIsReadonly(false);

        ClsesAndInstancesTab instTab = new ClsesAndInstancesTab();
        instTab.setup(mp_prj.createWidgetDescriptor(), mp_prj);
        instTab.initialize();

        return instTab;
    }

    @Override
    protected void addRefreshButton() {
        return;
    }
}
