/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protege.server.metaproject.impl;

import edu.stanford.smi.protege.exception.OntologyException;
import edu.stanford.smi.protege.model.Instance;
import edu.stanford.smi.protege.server.metaproject.PropertyValue;

public class PropertyValueImpl extends WrappedProtegeInstanceImpl implements
		PropertyValue {

	private static final long serialVersionUID = 7265423663403555058L;

	protected PropertyValueImpl(MetaProjectImpl mp, Instance pi) {
		super(mp, pi, MetaProjectImpl.ClsEnum.PropertyValue);
	}

	public String getPropertyName() {
		Object value = getProtegeInstance().getOwnSlotValue(getMetaProject().getSlot(MetaProjectImpl.SlotEnum.propertyName));
		if (!(value instanceof String)) {
			throw new OntologyException("The " + MetaProjectImpl.SlotEnum.propertyName + " slot should take on string values");
		}
		return (String) value;
	}

	public String getPropertyValue() {
		Object value = getProtegeInstance().getOwnSlotValue(getMetaProject().getSlot(MetaProjectImpl.SlotEnum.propertyValue));
		if (!(value instanceof String)) {
			throw new OntologyException("The " + MetaProjectImpl.SlotEnum.propertyValue + " slot should take on string values");
		}
		return (String) value;
	}

	public void setPropertyName(String name) {
		setSlotValue(MetaProjectImpl.SlotEnum.propertyName, name);
	}

	public void setPropertyValue(String value) {
		setSlotValue(MetaProjectImpl.SlotEnum.propertyValue, value);
	}

}
