/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protege.util;

/**
 * Class used for reporting errors at project load time. An instance of this class
 * contains an exception and a string message explaining the error.
 * @author Tania Tudorache <tudorache@stanford.edu>
 *
 */
public class MessageError {
    public enum Severity {
        WARNING, ERROR, FATAL;
    }
    
	private Throwable exception;
	private String message;
	private Severity severity;
	
	public MessageError(String message) {
		this(null, message);
	}
	
	public MessageError(Throwable ex) {
		this(ex,null);
	}
	
	public MessageError(Throwable ex, String message) {
	    this(ex,  message, Severity.WARNING);
	}
	
	public MessageError(Throwable ex, String message, Severity severity) {
		this.exception = ex;
		this.message = message;
		this.severity = severity;
	}

	public Throwable getException() {
		return exception;
	}

	public void setException(Throwable exception) {
		this.exception = exception;
	}

	public String getMessage() {
		return message;
	}

	public void setMessage(String message) {
		this.message = message;
	}
	
	public void setSeverity(Severity  severity) {
	    this.severity = severity;
	}
	
	public Severity getSeverity() {
	    return severity;
	}

}
