/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in 
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2007.  All Rights Reserved.
 *
 * Protege was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu.
 *
 */

package edu.stanford.smi.protege.util;

import java.io.*;
import java.net.*;
import java.util.*;

/**
 * 
 * @author Ray Fergerson <fergerson@smi.stanford.edu>
 */
public class MultiplexingClassLoader extends ClassLoader {
    private Collection childLoaders;

    public MultiplexingClassLoader(Collection childLoaders) {
        this.childLoaders = new ArrayList(childLoaders);
    }

    protected Class findClass(String name) throws ClassNotFoundException {
        Class clas = null;
        Iterator i = childLoaders.iterator();
        while (i.hasNext() && clas == null) {
            ClassLoader child = (ClassLoader) i.next();
            try {
                clas = child.loadClass(name);
            } catch (ClassNotFoundException e) {
                // do nothing
            }
        }
        if (clas == null) {
            clas = super.findClass(name);
        }
        return clas;
    }

    protected URL findResource(String name) {
        URL resource = null;
        Iterator i = childLoaders.iterator();
        while (i.hasNext() && resource == null) {
            ClassLoader child = (ClassLoader) i.next();
            resource = child.getResource(name);
        }
        if (resource == null) {
            resource = super.findResource(name);
        }
        return resource;
    }

    protected Enumeration findResources(String name) throws IOException {
        Collection resources = new LinkedHashSet();
        Iterator i = childLoaders.iterator();
        while (i.hasNext()) {
            ClassLoader child = (ClassLoader) i.next();
            Collection childResources = Collections.list(child.getResources(name));
            resources.addAll(childResources);
        }
        Collection superResources = Collections.list(super.findResources(name));
        resources.addAll(superResources);
        return Collections.enumeration(resources);
    }
}
